package com.fr.plugin.wechat.sync.schedule;

import com.fr.base.TableData;
import com.fr.cluster.core.ClusterNode;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.data.CustomRole;
import com.fr.decision.authority.data.User;
import com.fr.decision.webservice.service.user.UserMiddleRoleService;
import com.fr.file.TableDataConfig;
import com.fr.general.PropertiesUtils;
import com.fr.general.data.DataModel;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.wechat.sync.HttpUtil;
import com.fr.plugin.wechat.sync.dao.TagRoleService;
import com.fr.plugin.wechat.sync.entity.TagRoleEntity;
import com.fr.scheduler.job.FineScheduleJob;
import com.fr.script.Calculator;
import com.fr.stable.StringUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.third.org.apache.http.entity.StringEntity;
import com.fr.third.v2.org.quartz.JobExecutionContext;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Author fr.open
 * @Date 2021/7/29
 * @Description
 **/
public class SyncThread extends FineScheduleJob {

    private static final String tagList = "https://qyapi.weixin.qq.com/cgi-bin/tag/list?access_token=";

    private static final String TOKEN_URL = "https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid=ID&corpsecret=SECRET";

    private static final String DELETE_TAG_URL = "https://qyapi.weixin.qq.com/cgi-bin/tag/delete?access_token=ACCESS_TOKEN&tagid=TAGID";

    private static final String TAG_LIST = "https://qyapi.weixin.qq.com/cgi-bin/tag/get?access_token=ACCESS_TOKEN&tagid=TAGID";

    private static final String DELETE_USER_LIST = "https://qyapi.weixin.qq.com/cgi-bin/tag/deltagusers?access_token=ACCESS_TOKEN";

    private static final String ADD_USER_LIST = "https://qyapi.weixin.qq.com/cgi-bin/tag/addtagusers?access_token=ACCESS_TOKEN";

    private static final String ADD_TAG_URL = "https://qyapi.weixin.qq.com/cgi-bin/tag/create?access_token=ACCESS_TOKEN";

    private DataModel dataModel;

    public SyncThread() {
        String name = PropertiesUtils.getProperties("wechat").getProperty("name");
        FineLoggerFactory.getLogger().info("get table data config is {}", name);
        TableData tableData = TableDataConfig.getInstance().getTableData(name);
        dataModel = tableData.createDataModel(Calculator.createCalculator());
    }

    @Override
    public void run(JobExecutionContext jobExecutionContext, ClusterNode clusterNode) throws Exception {
        FineLoggerFactory.getLogger().info("start sync wechat users");
        String secret = PropertiesUtils.getProperties("wechat").getProperty("secret");
        String apid = PropertiesUtils.getProperties("wechat").getProperty("apid");
        String token = getToken(apid, secret);
        if (StringUtils.isBlank(token)) {
            return;
        }
        List<CustomRole> customRoles = AuthorityContext.getInstance().getCustomRoleController().find(QueryFactory.create());
        customRoles = customRoles.stream().filter(e -> !"superusers".equals(e.getName())).collect(Collectors.toList());
        FineLoggerFactory.getLogger().info("get role size is {}", customRoles.size());
        Map<String, CustomRole> map = customRoles.stream().collect(Collectors.toMap(e -> e.getId(), e -> e, (k1, k2) -> k2));
        Map<String, String> link = TagRoleService.getLinkMap();
        JSONArray tagList = getTagList(token);
        Set<String> exist = new HashSet<>();
        for (int i = 0; i < tagList.size(); i++) {
            JSONObject object = tagList.getJSONObject(i);
            String tagid = object.getString("tagid");
            String name = object.getString("tagname");
            //不存在的tag不进行处理
            String roleId = link.get(tagid);
            if (StringUtils.isBlank(roleId)) {
                continue;
            }
            //如果
            CustomRole role = map.get(roleId);
            if (role == null) {
                //删除当前标签
                FineLoggerFactory.getLogger().info("delete tag is {}", name);
                deleteTag(token, tagid);
                TagRoleService.delete(tagid);
                continue;
            }
            exist.add(roleId);
            //获取当前角色下的用户
            Set<String> users = getUsers(roleId);
            //获取当前标签下的用户
            Set<String> tagUserList = getTagUser(token, tagid);
            //待删除tag的用户
            Set<String> deleteUsers = tagUserList.stream().filter(e -> !users.contains(e)).collect(Collectors.toSet());
            FineLoggerFactory.getLogger().info("delete users {} to tag:{}",deleteUsers,name);
            deleteUser(token, tagid, deleteUsers);
            //待添加的用户
            Set<String> addUsers = users.stream().filter(e -> !tagUserList.contains(e)).collect(Collectors.toSet());
            FineLoggerFactory.getLogger().info("add users {} to tag:{}",addUsers,name);
            addUser(token, tagid, addUsers);
        }
        //角色中不存在的需要添加标签
        List<String> addTags = map.keySet().stream().filter(e -> !exist.contains(e)).collect(Collectors.toList());
        for (String role : addTags) {
            CustomRole customRole = map.get(role);
            String tagid = addTag(token, customRole.getName());
            TagRoleService.create(new TagRoleEntity(tagid, role));
            addUser(token, tagid, getUsers(role));
        }

    }

    private String addTag(String token, String name) {
        JSONObject object = JSONObject.create().put("tagname", name);
        StringEntity s = new StringEntity(object.toString(), "UTF-8");
        s.setContentEncoding("UTF-8");
        String res = HttpUtil.sendPost(ADD_TAG_URL.replace("ACCESS_TOKEN", token), null, object);
        FineLoggerFactory.getLogger().info("add tag res is {}", res);
        JSONObject resObj = new JSONObject(res);
        return resObj.getString("tagid");
    }


    private Set<String> getUsers(String roleId) throws Exception {
        Set<String> ids = UserMiddleRoleService.getInstance().getAllUserIdsByCustomRole(roleId);
        List<User> all = AuthorityContext.getInstance().getUserController().find(QueryFactory.create());
        Set<String> names = all.stream().filter(e -> ids.contains(e.getId())).map(e -> e.getUserName()).collect(Collectors.toSet());
        Set<String> userids = new HashSet<>();
        try {
            for (int i = 0; i < dataModel.getRowCount(); i++) {
                Object valueAt = dataModel.getValueAt(i, 4);
                if (valueAt != null && names.contains(valueAt.toString())) {
                    Object id = dataModel.getValueAt(i, 6);
                    if(id != null && StringUtils.isNotBlank(id.toString())){
                        userids.add(String.valueOf(id));
                    }
                }
            }
        } catch (Exception e) {
        }
        FineLoggerFactory.getLogger().info("get users ids {} by usernames {}",userids,names);
        return userids;
    }

    private void addUser(String token, String tagid, Set<String> addUsers) {
        if (addUsers == null || addUsers.isEmpty()) {
            return;
        }
        JSONObject object = JSONObject.create().put("tagid", Integer.valueOf(tagid));
        object.put("userlist", addUsers);
        String res = HttpUtil.sendPost(ADD_USER_LIST.replace("ACCESS_TOKEN", token), null, object);
        FineLoggerFactory.getLogger().info("add tag user res is {}", res);
    }

    private void deleteUser(String token, String tagid, Set<String> deleteUsers) {
        if (deleteUsers == null || deleteUsers.isEmpty()) {
            return;
        }
        JSONObject object = JSONObject.create().put("tagid", Integer.valueOf(tagid));
        object.put("userlist", deleteUsers);
        String res = HttpUtil.sendPost(DELETE_USER_LIST.replace("ACCESS_TOKEN", token), null, object);
        FineLoggerFactory.getLogger().info("delete tag user res is {}", res);
    }

    private Set<String> getTagUser(String token, String tagid) {
        String res = HttpUtil.sendGet(TAG_LIST.replace("ACCESS_TOKEN", token).replace("TAGID", tagid), null, null);
        FineLoggerFactory.getLogger().info("get tag user res is {}", res);
        JSONObject object = new JSONObject(res);
        JSONArray userlist = object.getJsonArray("userlist");
        Set<String> list = new HashSet<>();
        for (int i = 0; i < userlist.size(); i++) {
            JSONObject jsonObject = userlist.getJSONObject(i);
            list.add(jsonObject.getString("userid"));
        }
        return list;
    }

    /**
     * 删除tag
     *
     * @param token
     * @param tagid
     */
    private void deleteTag(String token, String tagid) {
        String res = HttpUtil.sendGet(DELETE_TAG_URL.replace("ACCESS_TOKEN", token).replace("TAGID", tagid), null, null);
        FineLoggerFactory.getLogger().info("delete tag res is {}", res);
    }

    private String getToken(String apid, String secret) {
        String res = HttpUtil.sendGet(TOKEN_URL.replace("ID", apid).replace("SECRET", secret), null, null);
        FineLoggerFactory.getLogger().info("get Token res is {}", res);
        JSONObject object = new JSONObject(res);
        return object.getString("access_token");
    }

    private JSONArray getTagList(String token) {
        String res = HttpUtil.sendGet(tagList + token, null, null);
        FineLoggerFactory.getLogger().info("get Tag res is {}", res);
        JSONObject object = new JSONObject(res);
        return object.getJsonArray("taglist");
    }
}
